@@ -1,4 +1,3 @@
-/* -*- Mode: javascript; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -21,6 +20,7 @@
  *
  * Contributor(s):
  *   Mike Shaver <shaver@off.net>
+ *   Daniel Boelzle <daniel.boelzle@sun.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -180,7 +180,7 @@ calAttendee.prototype = {
 
     get propertyEnumerator() { return this.mProperties.enumerator; },
 
-    // The has/get/getUnproxied/set/deleteProperty methods are case-insensitive.
+    // The has/get/set/deleteProperty methods are case-insensitive.
     getProperty: function (aName) {
         return this.mProperties.getProperty(aName.toUpperCase());
     },
@@ -198,7 +198,17 @@ calAttendee.prototype = {
     },
     set id(aId) {
         this.modify();
-        return this.mId = aId.replace(/^mailto:/i, "MAILTO:");
+        // RFC 1738 para 2.1 says we should be using lowercase mailto: urls
+        return (this.mId = (aId ? aId.replace(/^mailto:/i, "mailto:") : null));
+    },
+
+    toString: function calAttendee_toString() {
+        var ret = (this.id || "unknown").replace(/^mailto:/i, "");
+        var cn = this.commonName;
+        if (cn) {
+            ret = (cn + " <" + ret + ">");
+        }
+        return ret;
     }
 };
 
