@@ -151,12 +151,21 @@ CAL_VALUETYPE_ATTR(calDateTime, PRInt16, Hour)
 CAL_VALUETYPE_ATTR(calDateTime, PRInt16, Minute)
 CAL_VALUETYPE_ATTR(calDateTime, PRInt16, Second)
 CAL_VALUETYPE_ATTR(calDateTime, PRBool, IsDate)
-CAL_ISUPPORTS_ATTR(calDateTime, calITimezone, Timezone)
 CAL_VALUETYPE_ATTR_GETTER(calDateTime, PRBool, IsValid)
 CAL_VALUETYPE_ATTR_GETTER(calDateTime, PRTime, NativeTime)
 CAL_VALUETYPE_ATTR_GETTER(calDateTime, PRInt16, Weekday)
 CAL_VALUETYPE_ATTR_GETTER(calDateTime, PRInt16, Yearday)
 
+CAL_ISUPPORTS_ATTR_GETTER(calDateTime, calITimezone, Timezone)
+
+NS_IMETHODIMP calDateTime::SetTimezone(calITimezone *aValue) {
+    NS_ENSURE_FALSE(mImmutable, NS_ERROR_OBJECT_IS_IMMUTABLE);
+    NS_ENSURE_ARG_POINTER(aValue);
+    mTimezone = aValue;
+    CAL_ATTR_SET_POST;
+    return NS_OK;
+}
+
 NS_IMETHODIMP
 calDateTime::GetTimezoneOffset(PRInt32 *aResult)
 {
@@ -400,7 +409,7 @@ calDateTime::SetIcalString(nsACString const& aIcalString)
     if (icaltime_is_null_time(icalt)) {
         return calIErrors::ICS_ERROR_BASE + icalerrno;
     }
-    FromIcalTime(&icalt, mTimezone);
+    FromIcalTime(&icalt, nsnull);
     return NS_OK;
 }
 
@@ -471,11 +480,9 @@ void calDateTime::FromIcalTime(icaltimetype const* icalt, calITimezone * tz)
 #if defined(DEBUG)
     if (mTimezone) {
         if (t.is_utc) {
-            NS_ASSERTION(cal::sameXpcomObject(mTimezone, cal::UTC()),
-                         "UTC mismatch!");
+            NS_ASSERTION(SameCOMIdentity(mTimezone, cal::UTC()), "UTC mismatch!");
         } else if (!t.zone) {
-            NS_ASSERTION(cal::sameXpcomObject(mTimezone, cal::floating()),
-                         "floating mismatch!");
+            NS_ASSERTION(SameCOMIdentity(mTimezone, cal::floating()), "floating mismatch!");
         } else {
             nsCAutoString tzid;
             mTimezone->GetTzid(tzid);
