@@ -19,6 +19,7 @@
  *
  * Contributor(s):
  *   Michael Buettner <michael.buettner@sun.com>
+ *   Philipp Kewisch <mozilla@kewis.ch>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -139,7 +140,6 @@ function onLoad() {
 }
 
 function onAccept() {
-    var kDefaultTimezone = calendarDefaultTimezone();
     var attendees = document.getElementById("attendees-list");
     window.arguments[0].onOk(
         attendees.attendees,
@@ -235,8 +235,8 @@ function updateDateTime() {
         // the timezone of the endtime is "UTC", we convert
         // the endtime into the timezone of the starttime.
         if (startTime && endTime) {
-            if (startTime.timezone != endTime.timezone) {
-                if (endTime.timezone == "UTC") {
+            if (!compareObjects(startTime.timezone, endTime.timezone)) {
+                if (endTime.timezone.isUTC) {
                     endTime = endTime.getInTimezone(startTime.timezone);
                 }
             }
@@ -245,8 +245,8 @@ function updateDateTime() {
         // Before feeding the date/time value into the control we need
         // to set the timezone to 'floating' in order to avoid the
         // automatic conversion back into the OS timezone.
-        startTime.timezone = "floating";
-        endTime.timezone = "floating";
+        startTime.timezone = floating();
+        endTime.timezone = floating();
 
         document.getElementById("event-starttime").value = startTime.jsDate;
         document.getElementById("event-endtime").value = endTime.jsDate;
@@ -264,8 +264,8 @@ function updateDateTime() {
         // Before feeding the date/time value into the control we need
         // to set the timezone to 'floating' in order to avoid the
         // automatic conversion back into the OS timezone.
-        startTime.timezone = "floating";
-        endTime.timezone = "floating";
+        startTime.timezone = floating();
+        endTime.timezone = floating();
 
         document.getElementById("event-starttime").value = startTime.jsDate;
         document.getElementById("event-endtime").value = endTime.jsDate;
@@ -275,15 +275,6 @@ function updateDateTime() {
     updateAllDay();
 }
 
-function timezoneString(date) {
-    var fragments = date.split('/');
-    var num = fragments.length;
-    if (num <= 1) {
-        return fragments[0];
-    }
-    return fragments[num - 2] + '/' + fragments[num - 1];
-}
-
 /**
  * assumes that gStartDate and gEndDate have been correctly initialized,
  * either by having called loadDateTime() or having read the information
@@ -302,7 +293,7 @@ function updateTimezone() {
         var endTimezone = gEndTimezone;
         var equalTimezones = false;
         if (startTimezone && endTimezone &&
-            (startTimezone == endTimezone || endTimezone == "UTC")) {
+            (compareObjects(startTimezone, endTimezone) || endTimezone.isUTC)) {
             equalTimezones = true;
         }
 
@@ -310,7 +301,7 @@ function updateTimezone() {
         var tzEnd = document.getElementById("timezone-endtime");
         if (startTimezone != null) {
             tzStart.removeAttribute('collapsed');
-            tzStart.value = timezoneString(startTimezone);
+            tzStart.value = startTimezone.displayName || startTimezone.tzid;
         } else {
             tzStart.setAttribute('collapsed', 'true');
         }
@@ -318,7 +309,7 @@ function updateTimezone() {
         // we never display the second timezone if both are equal
         if (endTimezone != null && !equalTimezones) {
             tzEnd.removeAttribute('collapsed');
-            tzEnd.value = timezoneString(endTimezone);
+            tzEnd.value = endTimezone.displayName || endTimezone.tzid;
         } else {
             tzEnd.setAttribute('collapsed', 'true');
         }
@@ -346,19 +337,11 @@ function updateStartTime() {
     // jsDate is always in OS timezone, thus we create a calIDateTime
     // object from the jsDate representation and simply set the new
     // timezone instead of converting.
-    var kDefaultTimezone = calendarDefaultTimezone();
-    var start = jsDateToDateTime(startWidget.value);
-    start = start.getInTimezone(kDefaultTimezone);
-    if (gDisplayTimezone) {
-        start.timezone = gStartTimezone;
-    }
+    var start = jsDateToDateTime(startWidget.value,
+                                 gDisplayTimezone ? gStartTimezone : calendarDefaultTimezone());
     gStartDate = start.clone();
     start.addDuration(gDuration);
-    start = start.getInTimezone(gEndTimezone);
-    if (gDisplayTimezone) {
-        start.timezone = gEndTimezone;
-    }
-    gEndDate = start;
+    gEndDate = start.getInTimezone(gEndTimezone);
 
     var allDayElement = document.getElementById("all-day");
     var allDay = allDayElement.getAttribute("checked") == "true";
@@ -385,25 +368,17 @@ function updateEndTime() {
     var saveEndTime = gEndDate;
     var kDefaultTimezone = calendarDefaultTimezone();
 
-    var start = jsDateToDateTime(startWidget.value);
-    start = start.getInTimezone(kDefaultTimezone);
-    if (gDisplayTimezone) {
-        start.timezone = gStartTimezone;
-    }
-    gStartDate = start;
+    gStartDate = jsDateToDateTime(startWidget.value,
+                                  gDisplayTimezone ? gStartTimezone : calendarDefaultTimezone());
 
-    var end = jsDateToDateTime(endWidget.value);
-    end = end.getInTimezone(kDefaultTimezone);
     var timezone = gEndTimezone;
-    if (timezone == "UTC" &&
+    if (timezone.isUTC &&
         gStartDate &&
-        gStartTimezone != gEndTimezone) {
+        !compareObjects(gStartTimezone, gEndTimezone)) {
         timezone = gStartTimezone;
     }
-    if (gDisplayTimezone) {
-        end.timezone = timezone;
-    }
-    gEndDate = end;
+    gEndDate = jsDateToDateTime(endWidget.value,
+                                gDisplayTimezone ? timezone : kDefaultTimezone);
 
     var allDayElement = document.getElementById("all-day");
     var allDay = allDayElement.getAttribute("checked") == "true";
@@ -416,7 +391,7 @@ function updateEndTime() {
     // don't allow for negative durations.
     var warning = false;
     if (gEndDate.compare(gStartDate) >= 0) {
-        gDuration = end.subtractDate(start);
+        gDuration = gEndDate.subtractDate(gStartDate);
     } else {
         gStartDate = saveStartTime;
         gEndDate = saveEndTime;
@@ -452,13 +427,13 @@ function editStartTimezone() {
     args.time = gStartDate.getInTimezone(gStartTimezone);
     args.onOk = function(datetime) {
         var equalTimezones = false;
-        if (self.mStartTimezone && self.mEndTimezone &&
-            self.mStartTimezone == self.mEndTimezone) {
+        if (gStartTimezone && gEndTimezone &&
+            compareObjects(gStartTimezone, gEndTimezone)) {
             equalTimezones = true;
         }
-        self.mStartTimezone = datetime.timezone;
+        gStartTimezone = datetime.timezone;
         if (equalTimezones) {
-            self.mEndTimezone = datetime.timezone;
+            gEndTimezone = datetime.timezone;
         }
         self.propagateDateTime();
     };
@@ -481,15 +456,11 @@ function editEndTimezone() {
     var args = new Object();
     args.time = gEndTime.getInTimezone(gEndTimezone);
     args.onOk = function(datetime) {
-        var equalTimezones = false;
-        if (self.mStartTimezone && self.mEndTimezone &&
-            self.mStartTimezone == self.mEndTimezone) {
-            equalTimezones = true;
+        if (gStartTimezone && gEndTimezone &&
+            compareObjects(gStartTimezone, gEndTimezone)) {
+            gStartTimezone = datetime.timezone;
         }
-        if (equalTimezones) {
-            self.mStartTimezone = datetime.timezone;
-        }
-        self.mEndTimezone = datetime.timezone;
+        gEndTimezone = datetime.timezone;
         self.propagateDateTime();
     };
 
@@ -603,11 +574,8 @@ function onChangeCalendar(calendar) {
     // assume we're the organizer [in case that the calendar
     // does not support the concept of identities].
     gIsInvitation = false;
-
-    try {
-        gIsInvitation = provider.isInvitation(args.item);
-    }
-    catch (e) {
+    if (calInstanceOf(args.item.calendar, Components.interfaces.calISchedulingSupport)) {
+        gIsInvitation = args.item.calendar.isInvitation(args.item);
     }
 
     if (gIsReadOnly || gIsInvitation) {
@@ -736,7 +704,7 @@ function scrollToCurrentTime() {
 
 function setZoomFactor(aValue) {
     if (gZoomFactor == aValue) {
-        return val;
+        return aValue;
     }
 
     gZoomFactor = aValue;
@@ -786,7 +754,7 @@ function setForce24Hours(aValue) {
 
     var scrollbar = document.getElementById("horizontal-scrollbar");
     if (!scrollbar.hasAttribute("maxpos")) {
-        return;
+        return aValue;
     }
     var curpos = scrollbar.getAttribute("curpos");
     var maxpos = scrollbar.getAttribute("maxpos");
@@ -910,3 +878,27 @@ function onTimeChange(event) {
         grid.forceRefresh();
     }
 }
+
+/**
+ * This listener is used in sun-calendar-event-dialog-freebusy.xml inside the
+ * binding. It has been taken out of the binding to prevent leaks.
+ */
+function calFreeBusyListener(aFbElement, aBinding) {
+    this.mFbElement = aFbElement;
+    this.mBinding = aBinding;
+}
+
+calFreeBusyListener.prototype = {
+    onResult: function cFBL_onResult(aRequest, aEntries) {
+        if (aRequest && !aRequest.isPending) {
+            // Find request in list of pending requests and remove from queue:
+            function neq(aOp) {
+                return (aRequest.id != aOp.id);
+            }
+            this.mBinding.mPendingRequests = this.mBinding.mPendingRequests.filter(neq);
+        }
+        if (aEntries) {
+            this.mFbElement.onFreeBusy(aEntries);
+        }
+    }
+};
