@@ -14,7 +14,7 @@
  * The Original Code is Google Calendar Provider code.
  *
  * The Initial Developer of the Original Code is
- *   Philipp Kewisch (mozilla@kewis.ch)
+ *   Philipp Kewisch <mozilla@kewis.ch>
  * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
@@ -38,6 +38,54 @@
 // many events, the exact number is not important.
 const kMANY_EVENTS = 0x7FFFFFFF;
 
+function calGoogleSessionManager() {
+    this.wrappedJSObject = this;
+
+}
+
+calGoogleSessionManager.prototype = {
+    mSessionMap: {},
+
+    QueryInterface: function cGSM_QueryInterface(aIID) {
+        return doQueryInterface(this,
+                                calGoogleSessionManager.prototype,
+                                aIID,
+                                null,
+                                g_classInfo["calGoogleSessionManager"]);
+    },
+
+    /**
+     * getSessionByUsername
+     * Get a Session object for the specified username. If aCreate is false,
+     * null will be returned if the session doesn't exist. Otherwise, the
+     * session will be created.
+     *
+     * @param aUsername The username to get the session for
+     * @param aCreate   If true, the session will be created prior to returning
+     */
+    getSessionByUsername: function cGSM_getSessionByUsername(aUsername, aCreate) {
+        // If the username contains no @, assume @gmail.com
+        // XXX Maybe use accountType=GOOGLE and just pass the raw username?
+        if (aUsername.indexOf('@') == -1) {
+            aUsername += "@gmail.com";
+        }
+
+        // Check if the session exists
+        if (!this.mSessionMap.hasOwnProperty(aUsername)) {
+            if (!aCreate) {
+                return null;
+            }
+            LOG("Creating session for: " + aUsername);
+            this.mSessionMap[aUsername] = new calGoogleSession(aUsername);
+        } else {
+            LOG("Reusing session for: " + aUsername);
+        }
+
+        // XXX What happens if the username is "toSource" :)
+        return this.mSessionMap[aUsername];
+    }
+};
+
 /**
  * calGoogleSession
  * This Implements a Session object to communicate with google
@@ -49,26 +97,30 @@ function calGoogleSession(aUsername) {
 
     this.mItemQueue = new Array();
     this.mGoogleUser = aUsername;
+    this.wrappedJSObject = this;
 
     var username = { value: aUsername };
     var password = { value: null };
 
     // Try to get the password from the password manager
-    if (passwordManagerGet(aUsername, password)) {
+    if (aUsername && passwordManagerGet(aUsername, password)) {
         this.mGooglePass = password.value;
-        this.savePassword = true;
+        this.mPersistPassword = true;
         LOG("Retrieved Password for " + aUsername + " in constructor");
     }
+
+    // Register a freebusy provider for this session
+    getFreeBusyService().addProvider(this);
 }
 
 calGoogleSession.prototype = {
 
     QueryInterface: function cGS_QueryInterface(aIID) {
-        if (!aIID.equals(Components.interfaces.nsIInterfaceRequestor) &&
-            !aIID.equals(Components.interfaces.nsISupports)) {
-            throw Components.results.NS_ERROR_NO_INTERFACE;
-        }
-        return this;
+        return doQueryInterface(this,
+                                calGoogleSessionManager.prototype,
+                                aIID,
+                                null,
+                                g_classInfo["calGoogleSession"]);
     },
 
     /* Member Variables */
@@ -79,7 +131,7 @@ calGoogleSession.prototype = {
     mSessionID: null,
 
     mLoggingIn: false,
-    mSavePassword: false,
+    mPersistPassword: false,
     mItemQueue: null,
 
     mCalendarName: null,
@@ -89,55 +141,54 @@ calGoogleSession.prototype = {
      *
      * The auth token returned from Google Accounts
      */
-    get authToken() {
+    get authToken cGS_getAuthToken() {
         return this.mAuthToken;
     },
 
     /**
-     * attribute savePassword
+     * readonly attribute userName
      *
-     * Sets if the password for this user should be saved or not
+     * The username for this session. To get a session with a different
+     * username, use calIGoogleSessionManager.
      */
-    get savePassword() {
-        return this.mSavePassword;
-    },
-    set savePassword(v) {
-        return this.mSavePassword = v;
-    },
-
-    /**
-     * attribute googleFullName
-     *
-     * The Full Name of the user. If this is unset, it will return the
-     * this.googleUser, to ensure a non-zero value
-     */
-    get googleFullName() {
-        return (this.mGoogleFullName ? this.mGoogleFullName :
-                this.googleUser);
-    },
-    set googleFullName(v) {
-        return this.mGoogleFullName = v;
-    },
-
-    /**
-     * readonly attribute googleUser
-     *
-     * The username of the session. This does not necessarily have to be the
-     * email found in /calendar/feeds/email/private/full
-     */
-    get googleUser() {
+    get userName cGS_getUserName() {
         return this.mGoogleUser;
     },
 
     /**
-     * attribute googlePassword
+     * attribute persist
      *
-     * Sets the password used for the login process
+     * If set, the password will persist across restarts.
      */
-    get googlePassword() {
+    get persist cGS_getPersist() {
+        return this.mPersistPassword;
+    },
+    set persist cGS_setPersist(v) {
+        return this.mPersistPassword = v;
+    },
+
+    /**
+     * attribute AUTF8String fullName
+     *
+     * The user's full name, usually retrieved from the XML <author> fields.
+     */
+    get fullName cGS_getFullName() {
+        return this.mGoogleFullName;
+    },
+    set fullName cGS_setFullName(v) {
+        return this.mGoogleFullName = v;
+    },
+
+    /**
+     * attribute AUTF8String password
+     *
+     * The password used to authenticate. It is only important to implement the
+     * setter here, since the password is only used internally.
+     */
+    get password cGS_getPassword() {
         return this.mGooglePass;
     },
-    set googlePassword(v) {
+    set password cGS_setPassword(v) {
         return this.mGooglePass = v;
     },
 
@@ -148,10 +199,15 @@ calGoogleSession.prototype = {
     invalidate: function cGS_invalidate() {
         this.mAuthToken = null;
         this.mGooglePass = null;
+        this.persist = false;
 
         passwordManagerRemove(this.mGoogleUser);
     },
 
+    getCalendars: function cGS_getCalendars(aListener) {
+        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
+    },
+
     /**
      * failQueue
      * Fails all requests in this session's queue. Optionally only fail requests
@@ -163,8 +219,9 @@ calGoogleSession.prototype = {
     failQueue: function cGS_failQueue(aCode, aCalendar) {
         function cGS_failQueue_failInQueue(element, index, arr) {
             if (!aCalendar || (aCalendar && element.calendar == aCalendar)) {
-                element.fail(aCode);
+                element.fail(aCode, null);
                 return false;
+
             }
             return true;
         }
@@ -177,10 +234,9 @@ calGoogleSession.prototype = {
      * Prepares a login request, then requests via #asyncRawRequest
      *
      *
-     * @param     aRequest      The request that initiated the login
+     * @param aCalendar    The calendar of the request that initiated the login.
      */
-    loginAndContinue: function cGS_loginAndContinue(aRequest) {
-
+    loginAndContinue: function cGS_loginAndContinue(aCalendar) {
         if (this.mLoggingIn) {
             LOG("loginAndContinue called while logging in");
             return;
@@ -195,46 +251,46 @@ calGoogleSession.prototype = {
             // Check if we have a password. If not, authentication may have
             // failed.
             if (!this.mGooglePass) {
-                var username= { value: this.mGoogleUser };
+                var username = { value: this.mGoogleUser };
                 var password = { value: null };
-                var savePassword = { value: false };
+                var persist = { value: false };
 
                 // Try getting a new password, potentially switching sesssions.
-                var calendarName = (aRequest.calendar ?
-                                    aRequest.calendar.googleCalendarName :
+                var calendarName = (aCalendar ?
+                                    aCalendar.googleCalendarName :
                                     this.mGoogleUser);
 
                 if (getCalendarCredentials(calendarName,
                                            username,
                                            password,
-                                           savePassword)) {
+                                           persist)) {
 
                     LOG("Got the pw from the calendar credentials: " +
                         calendarName);
 
                     // If a different username was entered, switch sessions
 
-                    if (aRequest.calendar &&
-                        username.value != this.mGoogleUser) {
-
-                        var newSession = getSessionByUsername(username.value);
-                        newSession.googlePassword = password.value;
-                        newSession.savePassword = savePassword.value;
-                        setCalendarPref(aRequest.calendar,
+                    if (aCalendar && username.value != this.mGoogleUser) {
+                        var newSession = getGoogleSessionManager()
+                                         .getSessionByUsername(username.value,
+                                                               true);
+                        newSession.password = password.value;
+                        newSession.persist = persist.value;
+                        setCalendarPref(aCalendar,
                                         "googleUser",
                                         "CHAR",
                                         username.value);
 
                         // Set the new session for the calendar
-                        aRequest.calendar.session = newSession;
-                        LOG("Setting " + aRequest.calendar.name +
-                            "'s Session to " + newSession.googleUser);
+                        aCalendar.session = newSession;
+                        LOG("Setting " + aCalendar.name +
+                            "'s Session to " + newSession.userName);
 
                         // Move all requests by this calendar to its new session
                         function cGS_login_moveToSession(element, index, arr) {
-                            if (element.calendar == aRequest.calendar) {
+                            if (element.calendar == aCalendar) {
                                 LOG("Moving " + element.uri + " to " +
-                                    newSession.googleUser);
+                                    newSession.userName);
                                 newSession.asyncItemRequest(element);
                                 return false;
                             }
@@ -253,27 +309,32 @@ calGoogleSession.prototype = {
                     // changed. Just adapt the password from the dialog and
                     // continue.
                     this.mGooglePass = password.value;
-                    this.savePassword = savePassword.value;
+                    this.persist = persist.value;
                 } else {
                     LOG("Could not get any credentials for " +
                         calendarName + " (" +
                         this.mGoogleUser + ")");
 
+                    if (aCalendar) {
+                        // First of all, disable the calendar so no further login
+                        // dialogs show up.
+                        aCalendar.setProperty("disabled", true);
+
+                        // Unset the session in the requesting calendar, if the user
+                        // canceled the login dialog that also asks for the
+                        // username, then the session is not valid. This also
+                        // prevents multiple login windows.
+                        aCalendar.session = null;
+                    }
+
                     // The User even canceled the login prompt asking for
                     // the user. This means we have to fail all requests
                     // that belong to that calendar and are in the queue. This
                     // will also include the request that initiated the login
-                    // request, so that dosent need to be handled extra.
+                    // request, so that dosent need to be handled extra. If no
+                    // calendar was passed, fail all request in that queue
                     this.failQueue(Components.results.NS_ERROR_NOT_AVAILABLE,
-                                   aRequest.calendar);
-
-                    // Unset the session in the requesting calendar, if the user
-                    // canceled the login dialog that also asks for the
-                    // username, then the session is not valid. This also
-                    // prevents multiple login windows.
-                    if (aRequest.calendar)
-                        aRequest.calendar.session = null;
-
+                                   aCalendar);
                     return;
                 }
             }
@@ -295,8 +356,9 @@ calGoogleSession.prototype = {
             var request = new calGoogleRequest(this);
 
             request.type = request.LOGIN;
-            request.extraData = aRequest;
-            request.setResponseListener(this, this.loginComplete);
+            request.calendar = aCalendar;
+            request.responseListener = this;
+
             request.setUploadData("application/x-www-form-urlencoded",
                                   "Email=" + encodeURIComponent(this.mGoogleUser) +
                                   "&Passwd=" + encodeURIComponent(this.mGooglePass) +
@@ -307,19 +369,17 @@ calGoogleSession.prototype = {
         } catch (e) {
             // If something went wrong, reset the login state just in case
             this.mLoggingIn = false;
-            LOG({action:"Error Logging In",
-                 result: e.result,
-                 message: e.message});
+            LOG("Error Logging In: " + e);
 
             // If something went wrong, then this.loginComplete should handle
-            // the error. We don't need to take care of aRequest, since it is
-            // also in this.mItemQueue.
-            this.loginComplete(null, e.result, e.message);
+            // the error. We don't need to take care of the request that
+            // initiated the login, since it is also in the item queue.
+            this.onResult({ status: e.result}, e.message);
         }
     },
 
     /**
-     * loginComplete
+     * onResult (loginComplete)
      * Callback function that is called when the login request to Google
      * Accounts has finished
      *  - Retrieves the Authentication Token
@@ -327,48 +387,44 @@ calGoogleSession.prototype = {
      *  - Processes the Item Queue
      *
      * @private
-     * @param aRequest      The request object that initiated the login
-     * @param aStatus       The return status of the request.
-     * @param aResult       The (String) Result of the Request
+     * @param aOperation    The calIOperation that initiated the login
+     * @param aData         The (String) Result of the Request
      *                      (or an Error Message)
      */
-    loginComplete: function cGS_loginComplete(aRequest, aStatus, aResult) {
-
+     onResult: function cGS_onResult(aOperation, aData) {
         // About mLoggingIn: this should only be set to false when either
         // something went wrong or mAuthToken is set. This avoids redundant
         // logins to Google. Hence mLoggingIn is set three times in the course
         // of this function
 
-        if (!aResult || aStatus != Components.results.NS_OK) {
+        if (!aData || !Components.isSuccessCode(aOperation.status)) {
             this.mLoggingIn = false;
-            LOG("Login failed. Status: " + aStatus);
+            LOG("Login failed. Status: " + aOperation.status);
 
-            if (aStatus == kGOOGLE_LOGIN_FAILED) {
+            if (aOperation.status == kGOOGLE_LOGIN_FAILED &&
+                aOperation.reauthenticate) {
                 // If the login failed, then retry the login. This is not an
                 // error that should trigger failing the calICalendar's request.
-                // The login request's extraData contains the request object
-                // that triggered the login initially
-                this.loginAndContinue(aRequest.extraData);
+                this.loginAndContinue(aOperation.calendar);
             } else {
-                LOG("Failing queue with " + aStatus);
-                this.failQueue(aStatus);
+                LOG("Failing queue with " + aOperation.status);
+                this.failQueue(aOperation.status);
             }
         } else {
-            var start = aResult.indexOf("Auth=");
+            var start = aData.indexOf("Auth=");
             if (start == -1) {
                 // The Auth token could not be extracted
                 this.mLoggingIn = false;
                 this.invalidate();
 
                 // Retry login
-                this.loginAndContinue(aRequest.extraData);
+                this.loginAndContinue(aOperation.calendar);
             } else {
+                this.mAuthToken = aData.substring(start + 5, aData.length - 1);
 
-                this.mAuthToken = aResult.substring(start+5,
-                                                    aResult.length - 1);
                 this.mLoggingIn = false;
 
-                if (this.savePassword) {
+                if (this.persist) {
                     try {
                         passwordManagerSave(this.mGoogleUser,
                                             this.mGooglePass);
@@ -390,182 +446,6 @@ calGoogleSession.prototype = {
         }
     },
 
-    /**
-     * addItem
-     * Add a single item to google.
-     *
-     * @param   aCalendar               An instance of calIGoogleCalendar this
-     *                                  request belongs to. The event will be
-     *                                  added to this calendar.
-     * @param   aItem                   An instance of calIEvent to add
-     * @param   aResponseListener       The function in aCalendar to call at
-     *                                  completion.
-     * @param   aExtraData              Extra data to be passed to the response
-     *                                  listener
-     */
-    addItem: function cGS_addItem(aCalendar,
-                                  aItem,
-                                  aResponseListener,
-                                  aExtraData) {
-
-        var request = new calGoogleRequest(this);
-        var xmlEntry = ItemToXMLEntry(aItem,
-                                      this.mGoogleUser,
-                                      this.googleFullName);
-
-        request.type = request.ADD;
-        request.uri = aCalendar.fullUri.spec;
-        request.setUploadData("application/atom+xml; charset=UTF-8", xmlEntry);
-        request.setResponseListener(aCalendar, aResponseListener);
-        request.extraData = aExtraData;
-        request.calendar = aCalendar;
-
-        this.asyncItemRequest(request);
-    },
-
-    /**
-     * modifyItem
-     * Modify a single item from google.
-     *
-     * @param   aCalendar               An instance of calIGoogleCalendar this
-     *                                  request belongs to.
-     * @param   aOldItem                The instance of calIEvent before
-     *                                  modification.
-     * @param   aNewItem                The instance of calIEvent after
-     *                                  modification.
-     * @param   aResponseListener       The function in aCalendar to call at
-     *                                  completion.
-     * @param   aExtraData              Extra data to be passed to the response
-     *                                  listener
-     */
-    modifyItem: function cGS_modifyItem(aCalendar,
-                                        aOldItem,
-                                        aNewItem,
-                                        aResponseListener,
-                                        aExtraData) {
-
-        var request = new calGoogleRequest(this);
-
-        var xmlEntry = ItemToXMLEntry(aNewItem,
-                                      this.mGoogleUser,
-                                      this.googleFullName);
-
-        request.type = request.MODIFY;
-        request.uri = getItemEditURI(aOldItem);
-        request.setUploadData("application/atom+xml; charset=UTF-8", xmlEntry);
-        request.setResponseListener(aCalendar, aResponseListener);
-        request.extraData = aExtraData;
-        request.calendar = aCalendar;
-
-        this.asyncItemRequest(request);
-    },
-
-    /**
-     * deleteItem
-     * Delete a single item from google.
-     *
-     * @param   aCalendar               An instance of calIGoogleCalendar this
-     *                                  request belongs to.
-     *                                  belongs to.
-     * @param   aItem                   An instance of calIEvent to delete
-     * @param   aResponseListener       The function in aCalendar to call at
-     *                                  completion.
-     * @param   aExtraData              Extra data to be passed to the response
-     *                                  listener
-     */
-    deleteItem: function cGS_deleteItem(aCalendar,
-                                        aItem,
-                                        aResponseListener,
-                                        aExtraData) {
-
-        var request = new calGoogleRequest(this);
-
-        request.type = request.DELETE;
-        request.uri = getItemEditURI(aItem);
-        request.setResponseListener(aCalendar, aResponseListener);
-        request.extraData = aExtraData;
-        request.calendar = aCalendar;
-
-        this.asyncItemRequest(request);
-    },
-
-    /**
-     * getItem
-     * Get a single item from google.
-     *
-     * @param   aCalendar               An instance of calIGoogleCalendar this
-     *                                  request belongs to.
-     * @param   aId                     The ID of the item requested
-     * @param   aResponseListener       The function in aCalendar to call at
-     *                                  completion.
-     * @param   aExtraData              Extra data to be passed to the response
-     *                                  listener
-     */
-    getItem: function cGS_getItem(aCalendar,
-                                  aId,
-                                  aResponseListener,
-                                  aExtraData) {
-
-        var request = new calGoogleRequest(this);
-
-        request.type = request.GET;
-        request.uri = aCalendar.fullUri.spec + "/" + aId;
-        request.setResponseListener(aCalendar, aResponseListener);
-        request.extraData = aExtraData;
-        request.calendar = aCalendar;
-
-        this.asyncItemRequest(request);
-    },
-
-    /**
-     * getItems
-     * Get a Range of items from google.
-     *
-     * @param   aCalendar               An instance of calIGoogleCalendar this
-     *                                  request belongs to.
-     * @param   aCount                  The maximum number of items to return
-     * @param   aRangeStart             An instance of calIDateTime that limits
-     *                                  the start date.
-     * @param   aRangeEnd               An instance of calIDateTime that limits
-     *                                  the end date.
-     * @param   aItemReturnOccurrences  A boolean, wether to return single
-     *                                  occurrences or not.
-     * @param   aResponseListener       The function in aCalendar to call at
-     *                                  completion.
-     * @param   aExtraData              Extra data to be passed to the response
-     *                                  listener
-     */
-
-    getItems: function cGS_getItems(aCalendar,
-                                    aCount,
-                                    aRangeStart,
-                                    aRangeEnd,
-                                    aItemReturnOccurrences,
-                                    aResponseListener,
-                                    aExtraData) {
-
-        var rfcRangeStart = toRFC3339(aRangeStart);
-        var rfcRangeEnd = toRFC3339(aRangeEnd);
-
-        var request = new calGoogleRequest(this);
-
-        request.type = request.GET;
-        request.uri = aCalendar.fullUri.spec;
-        request.setResponseListener(aCalendar, aResponseListener);
-        request.extraData = aExtraData;
-        request.calendar = aCalendar;
-
-        // Request Parameters
-        request.addQueryParameter("max-results",
-                                  aCount ? aCount : kMANY_EVENTS);
-        request.addQueryParameter("singleevents",
-                                  aItemReturnOccurrences ? "true" : "false");
-        request.addQueryParameter("start-min", rfcRangeStart);
-        request.addQueryParameter("start-max", rfcRangeEnd);
-
-        this.asyncItemRequest(request);
-    },
-
     /**
      * asyncItemRequest
      * get or post an Item from or to Google using the Queue.
@@ -589,7 +469,7 @@ calGoogleSession.prototype = {
             // will be processed when the login is complete. Otherwise start
             // logging in.
             if (!this.mLoggingIn && this.mAuthToken == null) {
-                this.loginAndContinue(aRequest);
+                this.loginAndContinue(aRequest.calendar);
             }
         }
     },
@@ -608,5 +488,110 @@ calGoogleSession.prototype = {
 
         ASSERT(aRequest);
         aRequest.commit(this);
+    },
+
+    /**
+     * calIFreeBusyProvider Implementation
+     */
+    getFreeBusyIntervals: function cGS_getFreeBusyIntervals(aCalId,
+                                                            aRangeStart,
+                                                            aRangeEnd,
+                                                            aBusyTypes,
+                                                            aListener) {
+        if (aCalId.indexOf("@") < 0 || aCalId.indexOf(".") < 0) {
+            // No valid email, screw it
+            aListener.onResult(null, null);
+            return null;
+        }
+
+        // Requesting only a DATE returns items based on UTC. Therefore, we make
+        // sure both start and end dates include a time and timezone. This may
+        // not quite be what was requested, but I'd say its a shortcoming of
+        // rfc3339.
+        if (aRangeStart) {
+            aRangeStart = aRangeStart.clone();
+            aRangeStart.isDate = false;
+        }
+        if (aRangeEnd) {
+            aRangeEnd = aRangeEnd.clone();
+            aRangeEnd.isDate = false;
+        }
+
+        var rfcRangeStart = toRFC3339(aRangeStart);
+        var rfcRangeEnd = toRFC3339(aRangeEnd);
+
+        var request = new calGoogleRequest(this);
+
+        request.type = request.GET;
+        request.uri = "https://www.google.com/calendar/feeds/" +
+                      encodeURIComponent(aCalId.replace(/^mailto:/i, "")) +
+                      "/private/free-busy";
+        request.operationListener = aListener;
+        request.itemRangeStart = aRangeStart;
+        request.itemRangeEnd = aRangeEnd;
+        request.reauthenticate = false;
+
+        // Request Parameters
+        request.addQueryParameter("ctz", calendarDefaultTimezone().tzid);
+        request.addQueryParameter("max-results", kMANY_EVENTS);
+        request.addQueryParameter("singleevents", "true");
+        request.addQueryParameter("start-min", rfcRangeStart);
+        request.addQueryParameter("start-max", rfcRangeEnd);
+
+        var session = this;
+        request.responseListener = {
+            onResult: function cGS_getFreeBusyIntervals_onResult(aOperation, aData) {
+                session.getFreeBusyIntervals_response(aOperation,
+                                                      aData,
+                                                      aCalId,
+                                                      aRangeStart,
+                                                      aRangeEnd);
+            }
+        };
+
+        this.asyncItemRequest(request);
+        return request;
+    },
+
+    getFreeBusyIntervals_response: function getFreeBusyIntervals_response(aOperation,
+                                                                          aData,
+                                                                          aCalId,
+                                                                          aRangeStart,
+                                                                          aRangeEnd) {
+        // Prepare Namespaces
+        var gCal = new Namespace("gCal",
+                                 "http://schemas.google.com/gCal/2005");
+        var gd = new Namespace("gd", "http://schemas.google.com/g/2005");
+        var atom = new Namespace("", "http://www.w3.org/2005/Atom");
+        default xml namespace = atom;
+
+        if (aOperation.status == kGOOGLE_LOGIN_FAILED ||
+            !Components.isSuccessCode(aOperation.status)) {
+            aOperation.operationListener.onResult(aOperation, null);
+            return;
+        }
+
+        // A feed was passed back, parse it. Due to bug 336551 we need to
+        // filter out the <?xml...?> part.
+        var xml = new XML(aData.substring(38));
+        var timezoneString = xml.gCal::timezone.@value.toString() || "UTC";
+        var timezone = getTimezoneService().getTimezone(timezoneString);
+
+        // This line is needed, otherwise the for each () block will never
+        // be entered. It may seem strange, but if you don't believe me, try
+        // it!
+        xml.link.(@rel);
+
+        var intervals = [];
+        const fbtypes = Components.interfaces.calIFreeBusyInterval;
+        for each (var entry in xml.entry) {
+            var start =  fromRFC3339(entry.gd::when.@startTime.toString(), timezone);
+            var end = fromRFC3339(entry.gd::when.@endTime.toString(), timezone);
+            var interval = new calFreeBusyInterval(aCalId, fbtypes.BUSY, start, end);
+            LOGinterval(interval);
+            intervals.push(interval);
+        }
+
+        aOperation.operationListener.onResult(aOperation, intervals);
     }
 };
