@@ -63,7 +63,8 @@ calMemoryCalendar.prototype = {
 
     initMemoryCalendar: function() {
         this.mObservers = new calListenerBag(Components.interfaces.calIObserver);
-        this.mItems = { };
+        this.mItems = {};
+        this.mMetaData = new calPropertyBag();
     },
 
     //
@@ -84,6 +85,7 @@ calMemoryCalendar.prototype = {
     deleteCalendar: function mem_deleteCal(cal, listener) {
         cal = cal.wrappedJSObject;
         cal.mItems = {};
+        cal.mMetaData = new calPropertyBag();
 
         try {
             listener.onDeleteCalendar(cal, Components.results.NS_OK, null);
@@ -128,12 +130,11 @@ calMemoryCalendar.prototype = {
             aItem.id = getUUID();
 
         if (aItem.id == null) {
-            if (aListener)
-                aListener.onOperationComplete (this.superCalendar,
-                                               Components.results.NS_ERROR_FAILURE,
-                                               aListener.ADD,
-                                               aItem.id,
-                                               "Can't set ID on non-mutable item to addItem");
+            this.notifyOperationComplete(aListener,
+                                         Components.results.NS_ERROR_FAILURE,
+                                         Components.interfaces.calIOperationListener.ADD,
+                                         aItem.id,
+                                         "Can't set ID on non-mutable item to addItem");
             return;
         }
 
@@ -141,45 +142,27 @@ calMemoryCalendar.prototype = {
             if (this.relaxedMode) {
                 // we possibly want to interact with the user before deleting
                 delete this.mItems[aItem.id];
-            }
-            else {
-                if (aListener) {
-                    aListener.onOperationComplete (this.superCalendar,
-                                                   Components.interfaces.calIErrors.DUPLICATE_ID,
-                                                   aListener.ADD,
-                                                   aItem.id,
-                                                   "ID already exists for addItem");
-                }
+            } else {
+                this.notifyOperationComplete(aListener,
+                                             Components.interfaces.calIErrors.DUPLICATE_ID,
+                                             Components.interfaces.calIOperationListener.ADD,
+                                             aItem.id,
+                                             "ID already exists for addItem");
                 return;
             }
         }
 
         aItem.calendar = this.superCalendar;
-        var rec = aItem.recurrenceInfo;
-        if (rec) {
-            var exceptions = rec.getExceptionIds({});
-            for each (var exid in exceptions) {
-                var exception = rec.getExceptionFor(exid, false);
-                if (exception) {
-                    if (!exception.isMutable) {
-                        exception = exception.clone();
-                    }
-                    exception.calendar = this.superCalendar;
-                    rec.modifyException(exception);
-                }
-            }
-        }
-        
+
         aItem.makeImmutable();
         this.mItems[aItem.id] = aItem;
 
         // notify the listener
-        if (aListener)
-            aListener.onOperationComplete (this.superCalendar,
-                                           Components.results.NS_OK,
-                                           aListener.ADD,
-                                           aItem.id,
-                                           aItem);
+        this.notifyOperationComplete(aListener,
+                                     Components.results.NS_OK,
+                                     Components.interfaces.calIOperationListener.ADD,
+                                     aItem.id,
+                                     aItem);
         // notify observers
         this.mObservers.notify("onAddItem", [aItem]);
     },
@@ -194,13 +177,11 @@ calMemoryCalendar.prototype = {
 
         var this_ = this;
         function reportError(errStr, errId) {
-            if (aListener) {
-                aListener.onOperationComplete(this_.superCalendar,
-                                              errId ? errId : Components.results.NS_ERROR_FAILURE,
-                                              aListener.MODIFY,
-                                              aNewItem.id,
-                                              errStr);
-            }
+            this_.notifyOperationComplete(aListener,
+                                          errId ? errId : Components.results.NS_ERROR_FAILURE,
+                                          Components.interfaces.calIOperationListener.MODIFY,
+                                          aNewItem.id,
+                                          errStr);
             return null;
         }
 
@@ -209,10 +190,9 @@ calMemoryCalendar.prototype = {
             return reportError(null, "ID for modifyItem item is null");
         }
 
-        var modifiedItem = aNewItem.clone();
-        if (modifiedItem.parentItem != modifiedItem) {
-            modifiedItem.parentItem.recurrenceInfo.modifyException(modifiedItem);
-            modifiedItem = modifiedItem.parentItem;
+        var modifiedItem = aNewItem.parentItem.clone();
+        if (aNewItem.parentItem != aNewItem) {
+            modifiedItem.recurrenceInfo.modifyException(aNewItem, false);
         }
 
         if (this.relaxedMode) {
@@ -235,7 +215,7 @@ calMemoryCalendar.prototype = {
             var storedOldItem = this.mItems[aOldItem.id];
 
             // compareItems is not suitable here. See bug 418805.
-            if (storedOldItem.icalString != aOldItem.icalString) {
+            if (!compareItemContent(storedOldItem, aOldItem)) {
                 return reportError("old item mismatch in modifyItem");
             }
 
@@ -253,13 +233,11 @@ calMemoryCalendar.prototype = {
         modifiedItem.makeImmutable();
         this.mItems[modifiedItem.id] = modifiedItem;
 
-        if (aListener) {
-            aListener.onOperationComplete (this.superCalendar,
-                                           Components.results.NS_OK,
-                                           aListener.MODIFY,
-                                           modifiedItem.id,
-                                           modifiedItem);
-        }
+        this.notifyOperationComplete(aListener,
+                                     Components.results.NS_OK,
+                                     Components.interfaces.calIOperationListener.MODIFY,
+                                     modifiedItem.id,
+                                     modifiedItem);
 
         // notify observers
         this.mObservers.notify("onModifyItem", [modifiedItem, aOldItem]);
@@ -268,37 +246,47 @@ calMemoryCalendar.prototype = {
 
     // void deleteItem( in calIItemBase aItem, in calIOperationListener aListener );
     deleteItem: function (aItem, aListener) {
-        if (this.readOnly) 
-            throw Components.interfaces.calIErrors.CAL_IS_READONLY;
-        if (aItem.id == null || this.mItems[aItem.id] == null) {
-            if (aListener)
-                aListener.onOperationComplete (this.superCalendar,
-                                               Components.results.NS_ERROR_FAILURE,
-                                               aListener.DELETE,
-                                               aItem.id,
-                                               "ID is null or is from different calendar in deleteItem");
+        if (this.readOnly) {
+            this.notifyOperationComplete(aListener,
+                                         Components.interfaces.calIErrors.CAL_IS_READONLY,
+                                         Components.interfaces.calIOperationListener.DELETE,
+                                         aItem.id,
+                                         "Calendar is readonly");
+            return;
+        }
+        if (aItem.id == null) {
+            this.notifyOperationComplete(aListener,
+                                         Components.results.NS_ERROR_FAILURE,
+                                         Components.interfaces.calIOperationListener.DELETE,
+                                         aItem.id,
+                                         "ID is null in deleteItem");
             return;
         }
 
-        var oldItem = this.mItems[aItem.id];
-        if (oldItem.generation != aItem.generation) {
-            if (aListener)
-                aListener.onOperationComplete (this.superCalendar,
-                                               Components.results.NS_ERROR_FAILURE,
-                                               aListener.DELETE,
-                                               aItem.id,
-                                               "generation mismatch in deleteItem");
-            return;
+        var oldItem;
+        if (this.relaxedMode) {
+            oldItem = aItem;
+        } else {
+            oldItem = this.mItems[aItem.id];
+            if (oldItem.generation != aItem.generation) {
+                this.notifyOperationComplete(aListener,
+                                             Components.results.NS_ERROR_FAILURE,
+                                             Components.interfaces.calIOperationListener.DELETE,
+                                             aItem.id,
+                                             "generation mismatch in deleteItem");
+                return;
+            }
         }
+            
 
         delete this.mItems[aItem.id];
+        this.mMetaData.deleteProperty(aItem.id);
 
-        if (aListener)
-            aListener.onOperationComplete (this.superCalendar,
-                                           Components.results.NS_OK,
-                                           aListener.DELETE,
-                                           aItem.id,
-                                           aItem);
+        this.notifyOperationComplete(aListener,
+                                     Components.results.NS_OK,
+                                     Components.interfaces.calIOperationListener.DELETE,
+                                     aItem.id,
+                                     aItem);
         // notify observers
         this.mObservers.notify("onDeleteItem", [oldItem]);
     },
@@ -308,29 +296,29 @@ calMemoryCalendar.prototype = {
         if (!aListener)
             return;
 
-        if (aId == null ||
-            this.mItems[aId] == null) {
-            aListener.onOperationComplete(this.superCalendar,
-                                          Components.results.NS_ERROR_FAILURE,
-                                          aListener.GET,
-                                          null,
-                                          "IID doesn't exist for getItem");
+        if (aId == null || this.mItems[aId] == null) {
+            // querying by id is a valid use case, even if no item is returned:
+            this.notifyOperationComplete(aListener,
+                                         Components.results.NS_OK,
+                                         Components.interfaces.calIOperationListener.GET,
+                                         aId,
+                                         null);
             return;
         }
 
         var item = this.mItems[aId];
         var iid = null;
 
-        if (item instanceof Components.interfaces.calIEvent) {
+        if (isEvent(item)) {
             iid = Components.interfaces.calIEvent;
-        } else if (item instanceof Components.interfaces.calITodo) {
+        } else if (isToDo(item)) {
             iid = Components.interfaces.calITodo;
         } else {
-            aListener.onOperationComplete (this.superCalendar,
-                                           Components.results.NS_ERROR_FAILURE,
-                                           aListener.GET,
-                                           aId,
-                                           "Can't deduce item type based on QI");
+            this.notifyOperationComplete(aListener,
+                                         Components.results.NS_ERROR_FAILURE,
+                                         Components.interfaces.calIOperationListener.GET,
+                                         aId,
+                                         "Can't deduce item type based on QI");
             return;
         }
 
@@ -339,12 +327,11 @@ calMemoryCalendar.prototype = {
                                iid,
                                null, 1, [item]);
 
-        aListener.onOperationComplete (this.superCalendar,
-                                       Components.results.NS_OK,
-                                       aListener.GET,
-                                       aId,
-                                       null);
-
+        this.notifyOperationComplete(aListener,
+                                     Components.results.NS_OK,
+                                     Components.interfaces.calIOperationListener.GET,
+                                     aId,
+                                     null);
     },
 
     // void getItems( in unsigned long aItemFilter, in unsigned long aCount, 
@@ -365,16 +352,28 @@ calMemoryCalendar.prototype = {
         // filters
         //
 
+        var wantUnrespondedInvitations = ((aItemFilter & calICalendar.ITEM_FILTER_REQUEST_NEEDS_ACTION) != 0);
+        var superCal;
+        try {
+            superCal = this.superCalendar.QueryInterface(Components.interfaces.calISchedulingSupport);
+        } catch (exc) {
+            wantUnrespondedInvitations = false;
+        }
+        function checkUnrespondedInvitation(item) {
+            var att = superCal.getInvitedAttendee(item);
+            return (att && (att.participationStatus == "NEEDS-ACTION"));
+        }
+
         // item base type
         var wantEvents = ((aItemFilter & calICalendar.ITEM_FILTER_TYPE_EVENT) != 0);
         var wantTodos = ((aItemFilter & calICalendar.ITEM_FILTER_TYPE_TODO) != 0);
         if(!wantEvents && !wantTodos) {
             // bail.
-            aListener.onOperationComplete (this.superCalendar,
-                                           Components.results.NS_ERROR_FAILURE,
-                                           aListener.GET,
-                                           null,
-                                           "Bad aItemFilter passed to getItems");
+            this.notifyOperationComplete(aListener,
+                                         Components.results.NS_ERROR_FAILURE,
+                                         Components.interfaces.calIOperationListener.GET,
+                                         null,
+                                         "Bad aItemFilter passed to getItems");
             return;
         }
 
@@ -419,12 +418,18 @@ calMemoryCalendar.prototype = {
             if (itemReturnOccurrences && item.recurrenceInfo) {
                 var occurrences = item.recurrenceInfo.getOccurrences(
                     aRangeStart, aRangeEnd, aCount ? aCount - itemsFound.length : 0, {});
+                if (wantUnrespondedInvitations) {
+                    occurrences = occurrences.filter(checkUnrespondedInvitation);
+                }
                 if (!isEvent_) {
                     occurrences = occurrences.filter(checkCompleted);
                 }
                 itemsFound = itemsFound.concat(occurrences);
-            } else if ((isEvent_ || checkCompleted(item)) &&
+            } else if ((!wantUnrespondedInvitations || checkUnrespondedInvitation(item)) &&
+                       (isEvent_ || checkCompleted(item)) &&
                        checkIfInRange(item, aRangeStart, aRangeEnd)) {
+                // This needs fixing for recurring items, e.g. DTSTART of parent may occur before aRangeStart.
+                // This will be changed with bug 416975.
                 itemsFound.push(item);
             }
             if (aCount && itemsFound.length >= aCount) {
@@ -439,10 +444,29 @@ calMemoryCalendar.prototype = {
                                itemsFound.length,
                                itemsFound);
 
-        aListener.onOperationComplete (this.superCalendar,
-                                       Components.results.NS_OK,
-                                       aListener.GET,
-                                       null,
-                                       null);
+        this.notifyOperationComplete(aListener,
+                                     Components.results.NS_OK,
+                                     Components.interfaces.calIOperationListener.GET,
+                                     null,
+                                     null);
+    },
+
+    //
+    // calISyncCalendar interface
+    //
+    setMetaData: function memory_setMetaData(id, value) {
+        this.mMetaData.setProperty(id, value);
+    },
+    deleteMetaData: function memory_deleteMetaData(id) {
+        this.mMetaData.deleteProperty(id);
+    },
+    getMetaData: function memory_getMetaData(id) {
+        return this.mMetaData.getProperty(id);
+    },
+    getAllMetaData: function memory_getAllMetaData(out_count,
+                                                   out_ids,
+                                                   out_values) {
+        this.mMetaData.getAllProperties(out_ids, out_values);
+        out_count.value = out_ids.value.length;
     }
 };
